import network_constants as nc
from security.token_manager import TokenManager

import requests


def add_table(use_case: str, table_name: str):
    ''' Adds the use-case table with all the mappings as dict Internal -> External. '''
    jwt = TokenManager.getInstance().getToken()
    
    mapping = { c : c for c in [
        # mapping does not change any of the names for these properties
        'call_type', 
        'origin_call', 
        'origin_stand', 
        'taxi_id', 
        'timestamp', 
        'day_type', 
        'missing_data', 
        ] } 

    mapping['start_location_lat'] = 'ployline[0][0]'
    mapping['start_location_long'] = 'ployline[0][1]'
    mapping['end_location_lat'] = 'ployline[-1][0]'
    mapping['end_location_long'] = 'ployline[-1][1]'
    mapping["UniqueID"] = "trip_id"
    
    url = f"https://articonf1.itec.aau.at:30420/api/use-cases/{use_case}/tables"

    table = {
        "name": table_name,
        "mappings": mapping
    }

    response = requests.post(
            url,
            verify=False, 
            proxies = { "http":None, "https":None },
            headers = { "Authorization": f"Bearer {jwt}"},
            json = table
        )

    print(url+": "+str(response.status_code))


def add_layers(use_case:str, table_name: str):
    jwt = TokenManager.getInstance().getToken()

    layers = [
        {
            "name": "CallTypeLayer",
            "properties": [
                "UniqueID",
                "call_type",
            ],
            "cluster_properties": [
                "call_type"
            ]
        },
        {
            "name": "DayTypeLayer",
            "properties": [
                "UniqueID",
                "day_type",
            ],
            "cluster_properties": [
                "day_type"
            ]
        },
        {
            "name": "OriginCallLayer",
            "properties": [
                "UniqueID",
                "call_type",
                "origin_call",
            ],
            "cluster_properties": [
                "call_type",
                "origin_call",
            ]
        },
        {
            "name": "OriginStandLayer",
            "properties": [
                "UniqueID",
                "call_type",
                "origin_stand",
            ],
            "cluster_properties": [
                "call_type",
                "origin_stand",
            ]
        },
        {
            "name": "TaxiIdLayer",
            "properties": [
                "UniqueID",
                "taxi_id",
            ],
            "cluster_properties": [
                "taxi_id",
            ]
        },
        {
            "name": "StartLocationLayer",
            "properties": [
                "UniqueID",
                "start_location_lat",
                "start_location_long",
            ],
            "cluster_properties": [
                "start_location_lat",
                "start_location_long",
            ]
        },
        {
            "name": "EndLocationLayer",
            "properties": [
                "UniqueID",
                "end_location_lat",
                "end_location_long",
            ],
            "cluster_properties": [
                "end_location_lat",
                "end_location_long",
            ]
        },
    ]

    for layer in layers:
        # add basic info to each layer
        layer["use_case"] = use_case
        layer["table"] = table_name
        layer["properties"].append("timestamp") 

        url = f"https://articonf1.itec.aau.at:30420/api/layers"

        response = requests.post(
            url,
            verify=False, 
            proxies = { "http":None, "https":None },
            headers = { "Authorization": f"Bearer {jwt}"},
            json = layer
        )

        print(url+": "+str(response.status_code))


def main(use_case: str, table_name: str = "community-prediction-taxi"):
    add_table(use_case, table_name)
    add_layers(use_case, table_name)