import network_constants as nc
from security.token_manager import TokenManager

import requests


def add_table(use_case: str, table_name: str):
    ''' Adds the use-case table with all the mappings as dict Internal -> External. '''
    jwt = TokenManager.getInstance().getToken()
    
    mapping = { c : c for c in [
        # mapping does not change any of the names for these properties
        'trending_date', 
        'title', 
        'channel_title', 
        'category_id', 
        'category_name', 
        'publish_time', 
        'tags', 
        'views', 
        'likes', 
        'dislikes', 
        'comment_count', 
        'comments_disabled', 
        'ratings_disabled', 
        'video_error_or_removed', 
        'country_code', 
        'country_id', 
        'publish_timestamp',         
        ] } 

    mapping["UniqueID"] = "video_id"
    mapping["trend_delay"] = "trend_duration"
    mapping["timestamp"] = "trending_timestamp"
    
    url = f"https://articonf1.itec.aau.at:30420/api/use-cases/{use_case}/tables"

    table = {
        "name": table_name,
        "mappings": mapping
    }

    response = requests.post(
            url,
            verify=False, 
            proxies = { "http":None, "https":None },
            headers = { "Authorization": f"Bearer {jwt}"},
            json = table
        )

    print(url+": "+str(response.status_code))


def add_layers(use_case:str, table_name: str):
    jwt = TokenManager.getInstance().getToken()

    layers = [
        {
            "name": "CategoryLayer",
            "properties": [
                "UniqueID",
                "category_id",
                "category_name",
            ],
            "cluster_properties": [
                "category_id"
            ]
        },
        {
            "name": "ViewsLayer",
            "properties": [
                "UniqueID",
                "views",
            ],
            "cluster_properties": [
                "views"
            ]
        },
        {
            "name": "LikesLayer",
            "properties": [
                "UniqueID",
                "likes",
            ],
            "cluster_properties": [
                "likes"
            ]
        },
        {
            "name": "DislikesLayer",
            "properties": [
                "UniqueID",
                "dislikes",
            ],
            "cluster_properties": [
                "dislikes"
            ]
        },
        {
            "name": "CommentCountLayer",
            "properties": [
                "UniqueID",
                "comment_count",
            ],
            "cluster_properties": [
                "comment_count"
            ]
        },
        {
            "name": "CountryLayer",
            "properties": [
                "UniqueID",
                "country_code",
                "country_id",
            ],
            "cluster_properties": [
                "country_id"
            ]
        },
        {
            "name": "TrendDelayLayer",
            "properties": [
                "UniqueID",
                "trend_delay",
            ],
            "cluster_properties": [
                "trend_delay"
            ]
        },
    ]

    for layer in layers:
        # add basic info to each layer
        layer["use_case"] = use_case
        layer["table"] = table_name
        layer["properties"].append("timestamp") 

        url = f"https://articonf1.itec.aau.at:30420/api/layers"

        response = requests.post(
            url,
            verify=False, 
            proxies = { "http":None, "https":None },
            headers = { "Authorization": f"Bearer {jwt}"},
            json = layer
        )

        print(url+": "+str(response.status_code))


def main(use_case: str, table_name: str = "community-prediction-youtube"):
    add_table(use_case, table_name)
    add_layers(use_case, table_name)